# Rise Lambda Foundation

## Install

```bash
npm i rise-lambda-foundation
```

## Usage

### lambda.uuid

```js
const { uuid } = require('rise-lambda-foundation')
const myId = uuid()
```

### lambda.getPostData

```js
const { getPostData } = require('rise-lambda-foundation')
export const handler = async (input) => {
    const data = await getPostData(input)
}
```

### lambda.getUserId

```js
const { getUserId } = require('rise-lambda-foundation')
export const handler = async (input) => {
    const userId = await getUserId(input)
}
```

### lambda.http.success

```js
const { http } = require('rise-lambda-foundation')
export const handler = async (input) => {
    return http.success({
        id: 100
    })
}
```

### lambda.http.validationError

```js
const { http } = require('rise-lambda-foundation')
export const handler = async (input) => {
    const message = 'there was a problem on your end'
    return aws.utils.http.validationError(message)
}
```

### lambda.http.serviceError

```js
const { http } = require('rise-lambda-foundation')
export const handler = async (input) => {
    const message = 'there was a problem on our end'
    return aws.utils.http.serviceError(message)
}
```

### lambda.makeSchema

```js
const { makeSchema } = require('rise-lambda-foundation')
const inputSchema = makeSchema({
    foo: 'number!',
    bar: 'string',
    order: {
        foo: 'number!',
        bar: 'string'
    },
    items: [
        {
            id: 'string'
        }
    ]
})

module.exports.handler = async (e) => {
    const input = JSON.parse(e.body)
    const inputValidation = inputSchema.validate(input)

    if (!inputValidation.isValid) {
        return {
            statusCode: 400,
            data: JSON.stringify({
                errors: inputValidation.errors
            })
        }
    }
    return {
        statusCode: 200,
        data: JSON.stringify({
            status: 'success'
        })
    }
}
```
