const AWS = require('aws-sdk')
const Ajv = require('ajv')
const ajv = new Ajv()

module.exorts.makeSchema = function makeSchema(s) {
    let theSchema = {
        type: 'object',
        properties: {},
        required: [],
        additionalProperties: false
    }

    function addPrimitive(k, v, path) {
        if (v === 'string') {
            return {
                properties: {
                    [k]: { type: 'string' }
                },
                required: []
            }
        }
        if (v === 'string!') {
            return {
                properties: {
                    [k]: { type: 'string' }
                },
                required: [k]
            }
        }
        if (v === 'number') {
            return {
                properties: {
                    [k]: { type: 'integer' }
                },
                required: []
            }
        }
        if (v === 'number!') {
            return {
                properties: {
                    [k]: { type: 'integer' }
                },
                required: [k]
            }
        }
    }

    Object.keys(s).forEach((k) => {
        if (Array.isArray(s[k])) {
            Object.keys(s[k][0]).forEach((kk) => {
                const res = addPrimitive(kk, s[k][0][kk])

                theSchema = {
                    ...theSchema,
                    properties: {
                        ...theSchema.properties,
                        [k]: {
                            type: 'array',
                            items: {
                                type: 'object',
                                properties: {
                                    ...(theSchema.properties[k]?.properties ||
                                        {}),
                                    ...res.properties
                                },
                                required: [
                                    ...(theSchema.properties[k]?.required ||
                                        []),
                                    ...res.required
                                ],
                                additionalProperties: false
                            }
                        }
                    }
                }
            })
        } else if (typeof s[k] === 'object') {
            Object.keys(s[k]).forEach((kk) => {
                const res = addPrimitive(kk, s[k][kk])

                theSchema = {
                    ...theSchema,
                    properties: {
                        ...theSchema.properties,
                        [k]: {
                            type: 'object',
                            properties: {
                                ...(theSchema.properties[k]?.properties || {}),
                                ...res.properties
                            },
                            required: [
                                ...(theSchema.properties[k]?.required || []),
                                ...res.required
                            ],
                            additionalProperties: false
                        }
                    }
                }
            })
        } else {
            const res = addPrimitive(k, s[k])
            theSchema = {
                ...theSchema,
                properties: {
                    ...theSchema.properties,
                    ...res.properties
                },
                required: [...theSchema.required, ...res.required]
            }
        }
    })

    const validate = ajv.compile(theSchema)
    return {
        validate: (x) => {
            const res = validate(x)
            console.log(res)
            return {
                isValid: res,
                errors: res ? [] : validate.errors
            }
        }
    }
}

module.exports.getUserId = (event) => {
    if (!event) return null
    if (!event.requestContext) return null
    if (!event.requestContext.authorizer) return null
    if (!event.requestContext.authorizer.jwt) return null
    if (!event.requestContext.authorizer.jwt.claims) return null
    if (!event.requestContext.authorizer.jwt.claims.sub) return null
    return event.requestContext.authorizer.jwt.claims.sub
}

/**
 * Get post body
 */
module.exports.getPostData = (event) => {
    return JSON.parse(event.body)
}

module.exports.uuid = () => AWS.util.uuid.v4()

const headers = {
    'Content-Type': 'application/json',
    'Access-Control-Allow-Origin': '*',
    'Access-Control-Allow-Headers': 'Content-Type,Authorization,authorization',
    'Access-Control-Allow-Methods': 'OPTIONS,PUT,GET,POST'
}

module.exports.http = {
    success: (x) => ({
        statusCode: 200,
        body: JSON.stringify(x),
        headers
    }),
    validationError: (message) => ({
        statusCode: 400,
        body: JSON.stringify({ message }),
        headers
    }),
    serviceError: (message) => ({
        statusCode: 500,
        body: JSON.stringify({ message }),
        headers
    })
}
